package com.googlecode.mad.mvntools.m2eclipsegen;

import java.io.File;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.googlecode.mad.mvntools.common.Factory;
import com.googlecode.mad.mvntools.common.interfaces.IFileUtil;
import com.googlecode.mad.mvntools.common.interfaces.IMavenVersionUtil;

/**
 * - Apache License 2.0 - <br>
 * Generator for the m2eclipse settings artifacts of a workspace <br>
 * <br>
 * LastChangedBy: $Author: marthaler.worb@gmail.com $<br>
 * LastChangedDate: $Date: 2010-12-29 17:11:35 +0100 (Mi, 29 Dez 2010) $<br>
 * 
 * @author daniel marthaler
 * @version $Revision: 4 $
 * @since 27.12.2010
 * @goal generate
 */
public class M2EclipseGenerator extends AbstractMojo {
    
    /**
     * * The project dependencies. *
     * 
     * @parameter expression="${project.build.directory}"
     */
    private String                                buildDir;
    
    private final String[]                        defaultsources = { "src/main/resources", "src/test/resources", "src/main/jasperreports", "src/main/java", "src/main/java/src-gen",
            "src/main/java/src-domain", "src/main/java/src", "ejbModule", "connectorModule", "src", "src/test/java", "gen/src", "test" };
    
    /**
     * @parameter
     */
    private String[]                              sourcefolders;
    
    /**
     * @parameter default-value="${project}"
     */
    private org.apache.maven.project.MavenProject mavenProject;
    
    /**
     * @parameter default-value="1.5"
     */
    private String                                javaVersion;
    
    /**
     * @parameter default-value="false"
     */
    private boolean                               resolveWorkspaceProjects;
    
    protected IFileUtil                           fileUtil       = Factory.getInstance().getIFileUtil();
    
    protected IMavenVersionUtil                   mvnUtil        = Factory.getInstance().getIMavenVersionUtil();
    
    public void execute() throws MojoExecutionException {
        
        this.getLog().info(this.mvnUtil.getMavenString(this.mavenProject.getGroupId(), this.mavenProject.getArtifactId(), this.getClass()));
        
        final List<String> def = Arrays.asList(this.defaultsources);
        
        if (this.sourcefolders != null) {
            def.addAll(Arrays.asList(this.sourcefolders));
        }
        
        final StringBuilder b = new StringBuilder();
        
        // hinzufuegen der sourcen, die als solche existieren
        for (final String elem : def) {
            final File temp = new File(this.getRootPath() + elem);
            if (temp.exists() && elem.equals("src")) {
                final File temp2 = new File(this.getRootPath() + elem + "/main");
                if (temp2.exists()) {
                    continue;
                }
            } else if (temp.exists()) {
                b.append("  <classpathentry kind=\"src\" path=\"" + elem + "\"/>\n");
            }
        }
        
        if ("".equals(b.toString())) {
            b.append("  <classpathentry kind=\"src\" path=\"src\"/>\n");
        }
        
        // replace if there is a emty entry
        final String classpath = this.fileUtil.loadFileFromJar("classpath", this.getClass()).replace("[replace_me1]", b.toString()).replace("path=\"\"", "path=\"src\"");
        
        this.fileUtil.replaceFileWithNewContent(this.getRootPath() + "/" + ".classpath", classpath);
        
        final String project = this.fileUtil.loadFileFromJar("project", this.getClass()).replace("[replace_me1]", this.mavenProject.getArtifactId());
        this.fileUtil.replaceFileWithNewContent(this.getRootPath() + "/" + ".project", project);
        
        String mavenPrefs = this.fileUtil.loadFileFromJar("org.maven.ide.eclipse.prefs", this.getClass());
        mavenPrefs = mavenPrefs.replace("[replace_me2]", new Date().toString());
        
        if (this.resolveWorkspaceProjects) {
            mavenPrefs = mavenPrefs.replace("[replace_me1]", "true");
        } else {
            mavenPrefs = mavenPrefs.replace("[replace_me1]", "false");
        }
        
        String eclipsePrefs = this.fileUtil.loadFileFromJar("org.eclipse.jdt.core.prefs", this.getClass());
        eclipsePrefs = eclipsePrefs.replace("[replace_me1]", this.javaVersion);
        eclipsePrefs = eclipsePrefs.replace("[replace_me2]", new Date().toString());
        
        this.fileUtil.deleteFileOrFolder(this.getRootPath() + ".settings");
        
        // .settings order erstellen, wenn nicht vorhanden
        final File settings = new File(this.getRootPath() + "/" + ".settings");
        
        if (!settings.exists()) {
            settings.mkdir();
        }
        
        this.fileUtil.replaceFileWithNewContent(this.getRootPath() + "/" + ".settings/org.maven.ide.eclipse.prefs", mavenPrefs);
        
        this.fileUtil.replaceFileWithNewContent(this.getRootPath() + "/" + ".settings/org.eclipse.jdt.core.prefs", eclipsePrefs);
        
    }
    
    protected String getRootPath() {
        return this.buildDir.replace("target", "");
    }
    
}
